@@ -329,6 +329,15 @@ module NumRu
 	@@humane_message = t_or_f
       end
 
+      @@strftime_fmt = nil  # nil, or String to explicitly specify the format
+                            # for strftime used in []
+      def self.strftime_fmt=(fmt)
+        @@strftime_fmt = fmt
+      end
+      def self.strftime_fmt
+        @@strftime_fmt
+      end
+
       def initialize(cell=false,bare_index=false,name=nil)
 	 @init_fin = false         # true/false (true if initializatn finished)
 	 @name = name              # String
@@ -574,16 +583,16 @@ module NumRu
 			 # ^ take "round" to avoid round error in sec
 			 # (Note that %S below takes floor of seconds).
 		  datetime = since + (sec/86400.0)
-		  info_lost = datetime.strftime('%Y-%m-%d %H:%M:%S%z')
+		  info_lost = datetime.strftime(@@strftime_fmt || '%Y-%m-%d %H:%M:%S%z')
 		  info_lost_set = true
 		elsif Units[units] =~ Units['months since 1-1-1'] # year,month,pentad,..
 		  monun = Units['months']
 		  if /mon/ =~ stun || /year/ =~ stun
 		    datetime = since >> tun.convert( cval, monun ).round
-		    info_lost = datetime.strftime('%Y-%m')
+		    info_lost = datetime.strftime(@@strftime_fmt || '%Y-%m')
 		  else  # maybe pentad
 		    datetime = since >> tun.convert( cval, monun )
-		    info_lost = datetime.strftime('%Y-%m-%d')
+		    info_lost = datetime.strftime(@@strftime_fmt || '%Y-%m-%d')
 		  end
 		  info_lost_set = true
 		end
@@ -627,7 +636,7 @@ module NumRu
 	    if last >= 0
 	       last -= self.length   # last will be counted from the end
 	    end
-	    slicer = { first..last, step }
+	    slicer = { (first..last) => step }
 	    newax=Axis.new(false,@bare_index)  # always not a cell axis
          when NArray, Array
 	    newax=Axis.new(false,@bare_index)  # always not a cell axis
@@ -674,6 +683,8 @@ module NumRu
 	slicer = Array.new
 	ax = self.pos.val
 
+        units = self.pos.get_att('units')
+
 	case coord_cutter
 	when true
 	  slicer=true
@@ -683,6 +694,8 @@ module NumRu
 	  range_ary = [range.first, range.last]
 	  xmin = range_ary.min
 	  xmax = range_ary.max
+          xmin = UNumeric.from_date(xmin,units).val if xmin.class <= Date
+          xmax = UNumeric.from_date(xmax,units).val if xmax.class <= Date
 	  wh = ( (ax >= xmin) & (ax <= xmax) ).where
 	  if wh.length == 0
 	    raise "Range #{range} does not include any grid point in the axis '#{name}'"
@@ -690,9 +703,10 @@ module NumRu
 	  idmin = wh.min
 	  idmax = wh.max
 	  slicer=idmin..idmax
-	when Numeric
+	when Numeric, Date, DateTime
 	  # find the nearst point
 	  pt = coord_cutter       
+          pt = UNumeric.from_date(pt,units).val if pt.class <= Date
 	  dx = (ax-pt).abs
 	  minloc = _minloc_(dx)
 	  if conserve_rank
@@ -704,6 +718,7 @@ module NumRu
 	  # find the nearst points
 	  ary = coord_cutter
 	  slicer = ary.collect{ |pt|
+            pt = UNumeric.from_date(pt,units).val if pt.class <= Date
 	    dx = (ax-pt).abs
 	    minloc = _minloc_(dx)
 	  }
@@ -937,4 +952,7 @@ if $0 == __FILE__
    axcl = axcel_c.collect{|va| va*2}
    print "###\n"
    p axcl.class, axcl.each_varray{|va| p va}
+
+   axpt.pos.set_att("units","days since 2001-01-01")
+   p axpt.cut(DateTime.new(2001,1,4,12,0))
 end
