@@ -377,6 +377,16 @@
     RETURN VALUE
     * a GPhys
 
+---first3D
+    Returns a 3D subset selecting the first elements of 4th, 5th, ..
+    dimensions, i.e., self[true, true, true, 0, ...]. (For graphics)
+
+    ARGUMENTS
+    * (none)
+
+    RETURN VALUE
+    * a GPhys
+
 ---coerce(other)
     ((|You know what it is.|))
 
@@ -624,6 +634,10 @@ module NumRu
       end
 
       ## For graphics -->
+      def first3D
+	raise "rank less than 3" if rank < 3
+	self[true,true,*([0]*(rank-3))]
+      end
       def first2D
 	raise "rank less than 2" if rank < 2
 	self[true,true,*([0]*(rank-2))]
