@@ -1,6 +1,7 @@
 require "numru/gphys/gphys_netcdf_io"
 require "numru/gphys/gphys_grads_io"
 require "numru/gphys/gphys_grib_io"
+require "numru/gphys/gphys_nusdas_io"
 
 =begin
 =module NumRu::GPhys::IO
@@ -61,11 +62,13 @@ For example, (('GPhys::IO.open(file, name)')) simply calls
       (('path@varname[,dimname=pos1[:pos2[:thinning_intv]][,dimname=...]]'))
 
     RETURN VALUES
-    * An Array consisting of [file, var, slice, thinning], where
+    * An Array consisting of [file, var, slice, cut_slice, thinning], where
       * file (String) : path
       * var (String) : variable name
-      * slice (Array) : subset specifier in physical coordinate
-        to be used as (('GPhys#cut[slice]')).
+      * slice (Array) : subset specifier by the grid numbers
+        to be used as (('GPhys#[slice]')).
+      * cut_slice (Array) : subset specifier in physical coordinate
+        to be used as (('GPhys#cut[cut_slice]')).
       * thinning (Array) : additional subset specifier for thinning
         with uniform intervals if needed to be used (('GPhys#[thinning]'))
         after appling (('GPhys#cut')).
@@ -121,15 +124,19 @@ module NumRu
       NETCDF = "NETCDF"
       GRADS = "GRADS"
       GRIB = "GRIB"
+      NUSDAS = "NUSDAS"
       @@iomdl =     {NETCDF => GPhys::NetCDF_IO,
                      GRADS => GPhys::GrADS_IO,
-                     GRIB => GPhys::Grib_IO}
+                     GRIB => GPhys::Grib_IO,
+                     NUSDAS => GPhys::NuSDaS_IO}
       @@file_class = {NETCDF => NetCDF,
                      GRADS => GrADS_Gridded,
-                     GRIB => Grib}
+                     GRIB => Grib,
+                     NUSDAS => NuSDaS}
       @@nc_pattern = [/\.nc$/]
       @@grad_pattern = [/\.ctl$/]
-      @@grib_pattern = [/\.grib$/]
+      @@grib_pattern = [/\.grib$/, /\.grb$/]
+      @@nus_pattern = [/\.nus$/]
 
       def file2type(file)
  	case file
@@ -141,6 +148,8 @@ module NumRu
 	  return GRADS
 	when Grib
 	  return GRIB
+        when NuSDaS
+          return NUSDAS
 	when Regexp
 	  return NETCDF     # So far, only NetCDF_IO supports Regexp. 
 	when *@@nc_pattern
@@ -149,12 +158,15 @@ module NumRu
 	  return GRADS
 	when *@@grib_pattern
           return GRIB
+        when *@@nus_pattern
+          return NUSDAS
 	when String
 	  return NETCDF if /^http:\/\// =~ file   # assume a DODS URL
           return nil unless File.exist?(file)
           return NETCDF if NetCDF_IO.is_a_NetCDF?(file)
           return GRADS if GrADS_IO.is_a_GrADS?(file)
           return GRIB if Grib_IO.is_a_Grib?(file)
+          return NUSDAS if NuSDaS_IO.is_a_NuSDaS?(file)
 	end
         return nil
       end
@@ -167,7 +179,7 @@ module NumRu
 	@@file_class[ file2type(file) ]
       end
 
-      types = ['nc','grad','grib']
+      types = ['nc','grad','grib','nus']
       types.each{|c|
 	eval <<-EOS
         def add_#{c}_pattern(*regexps)
@@ -212,6 +224,7 @@ module NumRu
 	end
 	if /,/ =~ var
 	  slice = Hash.new
+	  cut_slice = Hash.new
 	  thinning = Hash.new
 	  var_descr = var.split(/,/)
 	  var = var_descr.shift
@@ -220,31 +233,42 @@ module NumRu
 	      dimname = $1
 	      subset = $2
 	      case subset
+	      when /\^(.*):(.*):(.*)/
+		slice[dimname] = ($1.to_i)..($2.to_i)
+		thinning[dimname] = {(0..-1) => $3.to_i}
+	      when /\^(.*):(.*)/
+		slice[dimname] = ($1.to_i)..($2.to_i)
+	      when /\^(.*)/
+		slice[dimname] = $1.to_i
 	      when /(.*):(.*):(.*)/
-		slice[dimname] = ($1.to_f)..($2.to_f)
-		thinning[dimname] = {0..-1,$3.to_i}
+		cut_slice[dimname] = ($1.to_f)..($2.to_f)
+		thinning[dimname] = {(0..-1) => $3.to_i}
 	      when /(.*):(.*)/
-		slice[dimname] = ($1.to_f)..($2.to_f)
+		cut_slice[dimname] = ($1.to_f)..($2.to_f)
 	      else
-		slice[dimname] = subset.to_f
+		cut_slice[dimname] = subset.to_f
 	      end
 	    else
 	      raise "invalid URL: variable subset specification error\n\n" + 
 		"URL format: " + GTURLfmt
 	    end
 	  end
+	  slice = nil if slice.length == 0
+	  cut_slice = nil if cut_slice.length == 0
 	  thinning = nil if thinning.length == 0
 	else
 	  slice = nil
+	  cut_slice = nil
 	  thinning = nil
 	end
-	[file, var, slice, thinning]
+	[file, var, slice, cut_slice, thinning]
       end   # def parse_gturl
 
       def open_gturl(gturl)
-	file, var, slice, thinning = GPhys::IO.parse_gturl(gturl)
+	file, var, slice, cut_slice, thinning = GPhys::IO.parse_gturl(gturl)
 	gp = GPhys::IO.open(file,var)
-	gp = gp.cut(slice) if slice
+	gp = gp[slice] if slice
+	gp = gp.cut(cut_slice) if cut_slice
 	gp = gp[thinning] if thinning
 	gp
       end   # def open_gturl
