@@ -416,12 +416,16 @@ module NumRu
 
       def write(file, gphys, name=nil)
         write_grid(file, gphys)
-	NetCDF_Conventions.add_history(file, "#{self}.write "+gphys.name)
+	NetCDF_Conventions.add_history(file, "#{File.basename($0)} wrote "+gphys.name)
 	VArrayNetCDF.write(file, gphys.data, name, gphys.axnames)
 	nil
       end
 
       def each_along_dims_write(gphyses, files, *loopdims, &block)
+	files = [files] if !files.is_a?(Array)
+	files.each do |fl|
+	  NetCDF_Conventions.add_history(fl, "#{File.basename($0)}")
+	end
 	IO_Common::each_along_dims_write(gphyses, files, loopdims, NetCDF_IO,
 					 &block)
       end
