@@ -319,9 +319,9 @@ module NumRu
 
     case(mode)
     when /^r/
-      @mode = 'r'
+      @mode = 'rb'
     when /^w/
-      @mode = 'w'
+      @mode = 'wb'
     else
       raise ArgumentError, "Unsupported IO mode: #{mode}"
     end
@@ -336,17 +336,18 @@ module NumRu
       "big_endian"=>nil,
       "little_endian"=>nil,
       "cray_32bit_ieee"=>nil,
+      "365_day_calendar"=>nil,
     }
 
     case(@mode)
-    when('r')
+    when('rb')
       if (File.exists?(ctlfilename))
         @ctlfile = File.open(ctlfilename, mode)
         parse_ctl
       else
         raise "File #{ctlfilename} does not exist."
       end
-    when('w')
+    when('wb')
       @ctlfile = File.open(ctlfilename, mode)
       
       @dimensions = []
@@ -547,48 +548,55 @@ EOS
       end
 
       # substitute DSET by the target file name
-      # and determine first record in the target file (="init")
+      # and determine first record in the target file (="init_xxxx")
 
-      init_year = target_date.year
-      init_month = target_date.month
-      init_day = target_date.day
-      init_hour = target_date.hour
-      init_min = target_date.min
+      init_year = start_date.year
+      init_month = start_date.month
+      init_day = start_date.day
+      init_hour = start_date.hour
+      init_min = start_date.min
 
       @dset_r = @dset.dup
       if ( @dset_r =~ /%y[24]/ )
+        init_year = target_date.year
         init_month = 1  if !( @dset_r =~ /%m[12]/ )
         init_day = 1    if !( @dset_r =~ /%d[12]/ )
         init_hour = 0   if !( @dset_r =~ /%h[12]/ )
         init_min = 0 if !( @dset_r =~ /%n2/ )
-        @dset_r.gsub!( /%y2/, sprintf("%02d",target_date.year%100) )
-        @dset_r.gsub!( /%y4/, sprintf("%04d",target_date.year) )
+        @dset_r.gsub!( /%y2/, sprintf("%02d",init_year%100) )
+        @dset_r.gsub!( /%y4/, sprintf("%04d",init_year) )
       end
       if ( @dset_r =~ /%m[12]/ )
+        init_month = target_date.month
         init_day = 1    if !( @dset_r =~ /%d[12]/ )
         init_hour = 0   if !( @dset_r =~ /%h[12]/ )
         init_min = 0 if !( @dset_r =~ /%n2/ )
-        @dset_r.gsub!( /%m1/, sprintf("%d",target_date.month) )
-        @dset_r.gsub!( /%m2/, sprintf("%02d",target_date.month) )
+        @dset_r.gsub!( /%m1/, sprintf("%d",init_month) )
+        @dset_r.gsub!( /%m2/, sprintf("%02d",init_month) )
       end
       if ( @dset_r =~ /%d[12]/ )
+        init_day = target_date.day
         init_hour = 0   if !( @dset_r =~ /%h[12]/ )
         init_min = 0 if !( @dset_r =~ /%n2/ )
-        @dset_r.gsub!( /%d1/, sprintf("%d",target_date.day) )
-        @dset_r.gsub!( /%d2/, sprintf("%02d",target_date.day) )
+        @dset_r.gsub!( /%d1/, sprintf("%d",init_day) )
+        @dset_r.gsub!( /%d2/, sprintf("%02d",init_day) )
       end
       if ( @dset_r =~ /%h[12]/ )
+        init_hour = target_date.hour
         init_min = 0 if !( @dset_r =~ /%n2/ )
-        @dset_r.gsub!( /%h1/, sprintf("%d",target_date.hour) )
-        @dset_r.gsub!( /%h2/, sprintf("%02d",target_date.hour) )
+        @dset_r.gsub!( /%h1/, sprintf("%d",init_hour) )
+        @dset_r.gsub!( /%h2/, sprintf("%02d",init_hour) )
       end
       if ( @dset_r =~ /%n2/ )
-        @dset_r.gsub!( /%n2/, sprintf("%02d",target_date.min) )
+        init_min = target_date.min
+        @dset_r.gsub!( /%n2/, sprintf("%02d",init_min) )
       end
 
       init_date = DateTime.new(init_year,init_month,init_day,init_hour,init_min)
       init_t = 0
+
       ary_time[0..t].to_a.each_with_index{ |time, idx| 
+
         case dimensions[3][:increment_units]
         when 'years'
           before_target_file = ((start_date >> (time*12)) < init_date)
@@ -926,7 +934,7 @@ EOS
       iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
       str_byte = map[:offset] + @dimensions[0][:len] * @dimensions[1][:len] * iblock
       if( @options["sequential"] )
-        str_byte += iblock*8 + 4
+        str_byte += iblock*2 + 4
       end
     else
       raise "Variable does not exist: "+name
