@@ -6,16 +6,43 @@
 =end
 require "narray_miss"
 require "date"
-require "parsedate"
+require "time"
 
 module NumRu
   module GribUtils
     require "numru/gphys/grib_params"
     private
+    def str2uint1(str)
+      return nil if str.length==0
+      return str[0]
+    end
+    def str2uint2(str)
+      return nil if str.length==0
+      return (str[0]<<8)+str[1]
+    end
+    def str2uint3(str)
+      return nil if str.length==0
+      return (str[0]<<16)+(str[1]<<8)+str[2]
+    end
+=begin
     def str2uint(str)
       return nil until str
       return str.unpack("H*")[0].hex
     end
+=end
+    def str2int1(str)
+      n = str2uint1(str)
+      return n && n > 127 ? 128-n : n
+    end
+    def str2int2(str)
+      n = str2uint2(str)
+      return n && n > 32767 ? 32768-n : n
+    end
+    def str2int3(str)
+      n = str2uint3(str)
+      return n && n > 8388607 ? 8388608-n : n
+    end
+=begin
     def str2int(str)
       return nil if str.length==0
       ary = str.unpack("H*")
@@ -27,6 +54,7 @@ module NumRu
       end
       return ary.join("").hex*fact
     end
+=end
     def uint2str(i,len)
       str = [sprintf("%0#{len*2}x",i)].pack("H*")
       str.length==len || raise("str length is not valid")
@@ -43,10 +71,10 @@ module NumRu
     end
     def float_value( str )
       str.length==4 || raise("str length must be 4")
-      sa = str2int( str[0..0] )
+      sa = str2int1( str[0,1] )
       s = (sa<0) ? -1 : 1
       a = sa.abs
-      b = str2uint( str[1..3] )
+      b = str2uint3( str[1,3] )
       return s*(2.0)**(-24.0)*b*(16.0)**(a-64.0)
     end
     def get_bits( str, nstep, nbits, sbu, ebu, type )
@@ -172,41 +200,30 @@ module NumRu
           obj.file = file
           obj.pos = file.pos
 
-          len = 8
-          is = GribIS.new(obj,file.read(len))
-          if !is.grib?
+          is = GribIS.new(obj,file.read(8))
+          unless is.grib?
             raise "This file is not Grib file (INITIAL SECTION)"
           end
           obj.is = is
 
-          len = str2uint(file.read(3))
-          pds = GribPDS.new(obj,file.read(len-3))
+          pds = GribPDS.new(obj,file.read(str2uint3(file.read(3))-3))
           obj.pds = pds
 
-          if pds.gds?
-            len = str2uint(file.read(3))
-            gds = GribGDS.new(obj,file.read(len-3))
-          else
-            gds = GribGDS.new(obj,nil)
-          end
+          str = pds.gds? ? file.read(str2uint3(file.read(3))-3) : nil
+          gds = GribGDS.new(obj,str)
           obj.gds = gds
 
-          if pds.bms?
-            len = str2uint(file.read(3))
-            bms = GribBMS.new(obj,file.read(len-3))
-          else
-            bms = GribBMS.new(obj,nil)
-          end
+          str = pds.bms? ? file.read(str2uint3(file.read(3))-3) : nil
+          bms = GribBMS.new(obj,str)
           obj.bms = bms
 
-          len = str2uint(file.read(3))
+          len = str2uint3(file.read(3))
           bds = GribBDS.new(obj,file.pos,len)
-          file.seek(len-3,IO::SEEK_CUR)
           obj.bds = bds
+          file.seek(len-3,IO::SEEK_CUR)
 
-          len = 4
-          es = GribES.new(obj,file.read(len))
-          if !es.grib?
+          es = GribES.new(obj,file.read(4))
+          unless es.grib?
             raise "This file is not Grib file (END SECTION)"
           end
           obj.es = es
@@ -258,7 +275,7 @@ module NumRu
       end
       def set_time(t)
         if String===t
-          ye,mo,da,ho,mi, = ParseDate.parsedate(t)
+          ye,mo,da,ho,mi, = Time.parse(t).to_a.reverse![4..-2]
         else
           if t.nil?
             ye = 0
@@ -339,7 +356,7 @@ module NumRu
         return @is[0..3]=="GRIB"
       end
       def total_length
-        return str2uint( @is[4,3] )
+        return str2uint3( @is[4,3] )
       end
       def update_total_length
         len = self.length+@sgm.pds.length+@sgm.gds.length+@sgm.bms.length+@sgm.bds.length+@sgm.es.length
@@ -347,7 +364,7 @@ module NumRu
         return len
       end
       def varsion
-        return str2uint( @is[7,1] )
+        return str2uint1( @is[7,1] )
       end
       def set_version(ver)
         @is[7..7] = uint2str(ver,1)
@@ -362,7 +379,8 @@ module NumRu
       include GribUtils
       private
       def param
-        nid = str2uint(@pds[5,1])
+        nid = str2uint1(@pds[5,1])
+        params = nil
         case version
         when 0
           params = PARAMS_0
@@ -380,6 +398,8 @@ module NumRu
           if cid==98 # ECMWF
             if version==128
               params = PARAMS_ECMWF_128
+            elsif version == 162
+              params = PARAMS_ECMWF_162
             end
           end
         end
@@ -399,7 +419,7 @@ module NumRu
         return ["unknown (#{nid})", nid.to_s, "-", nil]
       end
       def cid
-        return str2uint(@pds[1,1])
+        return str2uint1(@pds[1,1])
       end
       def initialize(sgm,str=nil)
         @sgm = sgm
@@ -410,7 +430,7 @@ module NumRu
         @pds.length+3
       end
       def version
-        return str2uint( @pds[0,1] )
+        return str2uint1( @pds[0,1] )
       end
       def set_version(ver)
         @pds[0..0] = uint2str(ver,1)
@@ -425,7 +445,7 @@ module NumRu
       end
       alias :center_id= :set_center_id
       def pid
-        return str2uint( @pds[2,1] )
+        return str2uint1( @pds[2,1] )
       end
       def set_pid(id)
         @pds[2..2] = uint2str(id,1)
@@ -433,7 +453,7 @@ module NumRu
       end
       alias :pid= :set_pid
       def gid
-        return str2uint( @pds[3,1] )
+        return str2uint1( @pds[3,1] )
       end
       def set_gid(id)
         @pds[3..3] = uint2str(id,1)
@@ -441,7 +461,7 @@ module NumRu
       end
       alias :gid= :set_gid
       def gds?
-        flag = str2uint( @pds[4,1] )
+        flag = str2uint1( @pds[4,1] )
         return (flag>>7)&1==1
       end
       def set_gds(gds)
@@ -460,7 +480,7 @@ module NumRu
       end
       alias :gds= :set_gds
       def bms?
-        flag = str2uint( @pds[4,1] )
+        flag = str2uint1( @pds[4,1] )
         return (flag>>6)&1==1
       end
       def set_bms(bms)
@@ -489,21 +509,17 @@ module NumRu
       end
       alias :name_id= :set_name_id
       def sname
-        zid = str2uint( @pds[6,1] )
+        zid = str2uint1( @pds[6,1] )
         sn = param[1]
         zt = Z_TYPES[zid]
         zn = zt ? zt[1] : zid.to_s
-        if zn=="level"
-          return sn
-        else
-          return sn+"_"+zn
-        end
+        return zn=="level" ? sn : sn+"_"+zn
       end
       def unit
         return param[2]
       end
       def z_type
-        zid = str2uint( @pds[6,1] )
+        zid = str2uint1( @pds[6,1] )
         zt = Z_TYPES[zid]
         if zt
           return zt[0]
@@ -518,7 +534,7 @@ module NumRu
       end
       alias :z_id= :set_z_id
       def z_sname
-        zid = str2uint( @pds[6,1] )
+        zid = str2uint1( @pds[6,1] )
         if zn = Z_TYPES[zid]
           return zn[1]
         else
@@ -527,8 +543,8 @@ module NumRu
         end
       end
       def z_value
-        type = str2uint( @pds[6,1] )
-        str = @pds[7..8]
+        type = str2uint1( @pds[6,1] )
+        str = @pds[7,2]
         ary = Z_LEVELS[type]
         if ary.nil?
           ary = []
@@ -536,15 +552,15 @@ module NumRu
           ary = ary.dup
         end
         if ary.length==1
-          ary[0] = ary[0].dup.update( {"value"=> str2uint(str)} )
+          ary[0] = ary[0].dup.update( {"value"=> str2uint2(str)} )
         elsif ary.length==2
-          ary[0] = ary[0].dup.update( {"value"=> str2uint(str[0,1])} )
-          ary[1] = ary[1].dup.update( {"value"=> str2uint(str[1,1])} )
+          ary[0] = ary[0].dup.update( {"value"=> str2uint1(str[0,1])} )
+          ary[1] = ary[1].dup.update( {"value"=> str2uint1(str[1,1])} )
         end
         return ary
       end
       def set_z_value(val)
-        type = str2uint( @pds[6,1] )
+        type = str2uint1( @pds[6,1] )
         ary = Z_LEVELS[type]
         val.length==ary.length || raise("length of val is not collect")
         if val.length==1
@@ -559,7 +575,7 @@ module NumRu
       end
       alias :z_value= :set_z_value
       def initial
-        return [str2uint( @pds[21,1] ), str2uint( @pds[9,1] ), str2uint( @pds[10,1] ), str2uint( @pds[11,1] ), str2uint( @pds[12,1] ), str2uint( @pds[13,1] )]
+        return [str2uint1( @pds[21,1] ), str2uint1( @pds[9,1] ), str2uint1( @pds[10,1] ), str2uint1( @pds[11,1] ), str2uint1( @pds[12,1] ), str2uint1( @pds[13,1] )]
       end
       def set_initial(year,month,day,hour,min)
         cent = year/100+1
@@ -574,14 +590,14 @@ module NumRu
       end
       alias :initial= :set_initial
       def date
-        d = Date.new((str2uint( @pds[21,1] )-1)*100+str2uint( @pds[9,1] ),
-                     str2uint( @pds[10,1] ),
-                     str2uint( @pds[11,1] ) )
-        h = [str2uint( @pds[12,1] ),str2uint( @pds[13,1] )]
+        d = Date.new((str2uint1( @pds[21,1] )-1)*100+str2uint1( @pds[9,1] ),
+                     str2uint1( @pds[10,1] ),
+                     str2uint1( @pds[11,1] ) )
+        h = [str2uint1( @pds[12,1] ),str2uint1( @pds[13,1] )]
         return [d,h]
       end
       def time_unit
-        return TIME_UNITS[ str2uint( @pds[14,1] ) ]
+        return TIME_UNITS[ str2uint1( @pds[14,1] ) ]
       end
       def set_time_unit_id(id)
         @pds[14,1] = uint2str(id,1)
@@ -589,7 +605,7 @@ module NumRu
       end
       alias :time_unit_id= :set_time_unit_id
       def p1
-        return str2uint( @pds[15,1] )
+        return str2uint1( @pds[15,1] )
       end
       def set_p1(i)
         @pds[15,1] = uint2str(i,1)
@@ -597,7 +613,7 @@ module NumRu
       end
       alias :p1= :set_p1
       def p2
-        return str2uint( @pds[16,1] )
+        return str2uint1( @pds[16,1] )
       end
       def set_p2(i)
         @pds[16,1] = uint2str(i,1)
@@ -605,7 +621,7 @@ module NumRu
       end
       alias :p2= :set_p2
       def time_range
-        return str2uint( @pds[17,1] )
+        return str2uint1( @pds[17,1] )
       end
       def set_time_range_id(id)
         @pds[17,1] = uint2str(id,1)
@@ -613,7 +629,7 @@ module NumRu
       end
       alias :time_range_id= :set_time_range_id
       def ave
-        return str2uint( @pds[18,2] )
+        return str2uint2( @pds[18,2] )
       end
       def set_ave(val)
         @pds[18,2] = uint2str(val,2)
@@ -621,7 +637,7 @@ module NumRu
       end
       alias :ave= :set_ave
       def miss
-        return str2uint( @pds[20,1] )
+        return str2uint1( @pds[20,1] )
       end
       def set_miss(val)
         @pds[20,1] = uint2str(val,1)
@@ -629,7 +645,7 @@ module NumRu
       end
       alias :miss= :set_miss
       def sub_center
-        return str2uint( @pds[22,1] )
+        return str2uint1( @pds[22,1] )
       end
       def set_sub_center_id(id)
         @pds[22,1] = uint2str(id,1)
@@ -637,7 +653,7 @@ module NumRu
       end
       alias :sub_center_id= :set_sub_center_id
       def dfact
-        return str2int( @pds[23,2] )
+        return str2int2( @pds[23,2] )
       end
       def set_dfact(val)
         @pds[23..24] = int2str(val,2)
@@ -652,15 +668,15 @@ module NumRu
       include GribUtils
       private
       def gtype
-        return str2uint( @gds[2,1] )
+        return str2uint1( @gds[2,1] )
       end
       def get_x_y(grid)
       case gtype
         when 0,4
           #lon_lat (+GussLat)
-          flag = str2uint( grid[10,1] )
+          flag = str2uint1( grid[10,1] )
           flag_inc = (flag>>7)&1
-          flag = str2uint( grid[21,1] )
+          flag = str2uint1( grid[21,1] )
           flag_scani = (flag>>7)&1
           flag_scanj = (flag>>6)&1
           flag_ij = (flag>>5)&1
@@ -669,11 +685,11 @@ module NumRu
           lon["short_name"] = "lon"
           lon["long_name"] = "longitude"
           lon["units"] = "degrees_east"
-          nlon = str2uint( grid[0,2] )
+          nlon = str2uint2( grid[0,2] )
           nlon=="ffff".hex && raise("not defined yet")
-          lo1 = str2int( grid[7,3] )
-          lo2 = str2int( grid[14,3] )
-          dlon = str2uint( grid[17,2] )
+          lo1 = str2int3( grid[7,3] )
+          lo2 = str2int3( grid[14,3] )
+          dlon = str2uint2( grid[17,2] )
           if (!flag_inc)&&dlon==-"7fff".hex
             dlon = (lo2-lo1)/(nlon-1)
           else
@@ -698,11 +714,11 @@ module NumRu
           lat["short_name"] = "lat"
           lat["long_name"] = "latitude"
           lat["units"] = "degrees_north"
-          nlat = str2uint( grid[2,2] )
+          nlat = str2uint2( grid[2,2] )
           nlat=="ffff".hex && raise("not defined yet")
-          la1 = str2int( grid[4,3] )
-          la2 = str2int( grid[11,3] )
-          dlat = str2uint( grid[19,2] )
+          la1 = str2int3( grid[4,3] )
+          la2 = str2int3( grid[11,3] )
+          dlat = str2uint2( grid[19,2] )
           if gtype==0
             if (!flag_inc)&&dlat==-"7fff".hex
               dlat = (la2-la1)/(nlat-1)
@@ -836,15 +852,15 @@ module NumRu
         end
       end
       def nv
-        return str2uint( @gds[0,1] )
+        return str2uint1( @gds[0,1] )
       end
       def pv
-        pv = str2uint( @gds[1,1] )
+        pv = str2uint1( @gds[1,1] )
         return nil if pv==255 || nv==0
         return pv
       end
       def pl
-        pl = str2uint( @gds[1,1] )
+        pl = str2uint1( @gds[1,1] )
         return nil if pl==255
         return nv*4+pl
       end
@@ -953,7 +969,7 @@ module NumRu
         end
       end
       def map_number
-        nmap = str2uint( @bms[1,2] )
+        nmap = str2uint2( @bms[1,2] )
         nmap = false if nmap==0
         return nmap
       end
@@ -968,7 +984,7 @@ module NumRu
       end
       def map
         return nil unless has_map?
-        return get_bits( @bms[3..-1], 1,1, 0,str2uint(@bms[0,1]), NArray::BYTE )
+        return get_bits( @bms[3..-1], 1,1, 0,str2uint1(@bms[0,1]), NArray::BYTE )
       end
       def set_map(mask)
         map,n = data2str(mask,1,1)
@@ -987,14 +1003,14 @@ module NumRu
       def oct4
         return @oct4 if @oct4
         @sgm.file.seek(@pos, IO::SEEK_SET)
-        @oct4 = str2uint( @sgm.file.read(1) ) 
+        @oct4 = str2uint1( @sgm.file.read(1) ) 
         return @oct4
       end
       def oct14
         return nil if flag4==0
         return @oct14 if @oct14
         @sgm.file.seek(@pos+10, IO::SEEK_SET)
-        @oct14 = str2uint( @sgm.file.read(1) )
+        @oct14 = str2uint1( @sgm.file.read(1) )
         return @oct14
       end
       def flag1
@@ -1156,13 +1172,13 @@ module NumRu
       def efactor
         return @efactor if @efactor
         @sgm.file.seek(@pos+1, IO::SEEK_SET)
-        @efactor = 2.0**str2int( @sgm.file.read(2) )
+        @efactor = 2.0**str2int2( @sgm.file.read(2) )
         return @efactor
       end
       def value(shape,*arg)
         nbits_unuse = oct4&15
         @sgm.file.seek(@pos+7, IO::SEEK_SET)
-        nbits_pack = str2uint( @sgm.file.read(1) )
+        nbits_pack = str2uint1( @sgm.file.read(1) )
         if (nbits_pack%8) == 0
           nstep = 8
         elsif (nbits_pack%4) == 0
@@ -1223,11 +1239,11 @@ module NumRu
           return var
         else
           raise "not defined yet"
-          n1 = str2uint( @file.read(2) )
+          n1 = str2uint2( @file.read(2) )
           @sgm.file.seek(1, IO::SEEK_CUR)
-          n2 = str2uint( @sgm.file.read(2) )
-          p1 = str2uint( @sgm.file.read(2) )
-          p2 = str2uint( @sgm.file.read(2) ) # @bds[15..16]
+          n2 = str2uint2( @sgm.file.read(2) )
+          p1 = str2uint2( @sgm.file.read(2) )
+          p2 = str2uint2( @sgm.file.read(2) ) # @bds[15..16]
           #        width = str2uint( @bds[18..??] )
           #        bitmap = str2uint( @bds[??+1..n1-4] )
         end
@@ -1615,7 +1631,7 @@ module NumRu
           return false
         end
         if pds1.unit!=pds2.unit
-        $stderr.printf("unit is not same: #{pds1.unit} != #{pds2.unit}\n")
+          $stderr.printf("unit is not same: #{pds1.unit} != #{pds2.unit}\n")
           return false
         end
         zt1 = pds1.z_type
@@ -1743,10 +1759,14 @@ module NumRu
           else
             va.put_att("level", pds.z_type)
           end
-          time = Array.new
-          sgms.each{|sgm|
-            time.push( get_time(sgm.pds.date) )
+          len = sgms.length
+          time = Array.new(len)
+          len.times{|i|
+            time[i] = get_time(sgms[i].pds.date)
           }
+#          sgms.each{|sgm|
+#            time.push( get_time(sgm.pds.date) )
+#          }
           time.uniq!
           if time.length>1
             time = NArray.to_na(time)
