@@ -315,7 +315,7 @@ PARAM_HPBL = 2221
 PARAM_5WAVH = 2222
 PARAM_5WAVA = 2230
 
-## ECMWF
+## ECMWF 128
 PARAM_EQPT_TEMP = 3004
 PARAM_SEPT_TEMP = 3005
 PARAM_U_DIV_WND = 3011
@@ -449,6 +449,13 @@ PARAM_TND_ZWND_AD = 3253
 PARAM_TND_MWND_AD = 3254
 
 
+## ECMWF 128
+PARAM_WATV_EFLX = 4071
+PARAM_WATV_NFLX = 4072
+PARAM_DIV_MST_FLX = 4084
+
+
+
 NAMES_UNITS = {
   PARAM_UNKNOWN => ["unknown", "dat", nil, nil],
   PARAM_PRESSURE => ["Pressure", "PRES", "Pa" ,"air_pressure"],
@@ -849,8 +856,14 @@ NAMES_UNITS = {
 #  252 => PARAM_TND_HUM_AD,
 #  253 => PARAM_TND_ZWND_AD,
 #  254 => PARAM_TND_MWND_AD
-nil => nil
 
+## ECMWF 162  #incomplite
+  PARAM_WATV_EFLX => ["Vertical integral of eastward water vapour flux", "VIEWVF", "kg/m/s", nil],
+  PARAM_WATV_NFLX => ["Vertical integral of northward water vapour flux", "VINWVF", "kg/m/s", nil],
+  PARAM_DIV_MST_FLX => ["Vertical integral of divergence of moisture flux", "VIDMOF", "kg/m2/s", nil],
+
+
+nil => nil
 }
 
 
@@ -1248,3 +1261,10 @@ PARAMS_ECMWF_128 = {   #inconmplte
   253 => PARAM_TND_ZWND_AD,
   254 => PARAM_TND_MWND_AD
 }
+
+
+PARAMS_ECMWF_162 = {   #inconmplte
+  71 => PARAM_WATV_EFLX,
+  72 => PARAM_WATV_NFLX,
+  84 => PARAM_DIV_MST_FLX
+}
