@@ -131,7 +131,9 @@ A class to handle discretized grids of physical quantities.
        gphys.cut({'lon'=>135.5,'lat'=>0..20})
 
     RETURN VALUE
-    * a Grid
+    * an Array : [a Grid, slicer], where slicer is an array
+      to be used to make a subset of an corresponding varray
+      (to be used in GPhys#cut).
    
 ---cut_rank_conserving(*args)
     Similar to ((<cut>)), but the rank is conserved by not eliminating
@@ -307,7 +309,7 @@ module NumRu
 	  end
 	}
 
-	newgrid = Grid.new( *newaxes )
+	newgrid = self.class.new( *newaxes )
 	newgrid.set_lost_axes( @lost_axes.dup )
 
 	if !deleted_by 
@@ -329,7 +331,7 @@ module NumRu
 
       def copy
 	 # deep clone onto memory
-	 out = Grid.new( *@axes.collect{|ax| ax.copy} )
+	 out = self.class.new( *@axes.collect{|ax| ax.copy} )
 	 out.set_lost_axes( @lost_axes.dup )
 	 out
       end
@@ -349,8 +351,8 @@ module NumRu
 	    axes[i] = @axes[i]
 	  end
 	end
-	out = Grid.new( *axes )
-	out.set_lost_axes( @lost_axes.dup + other.lost_axes )
+	out = self.class.new( *axes )
+	out.set_lost_axes( (@lost_axes.dup + other.lost_axes).uniq )
 	out
       end
 
@@ -364,7 +366,7 @@ module NumRu
 	    # make a clone
 	    axes = Array.new
 	    (0...rank).each{ |i| axes.push( @axes[i][0..-1] ) }
-	    Grid.new( *axes )
+	    self.class.new( *axes )
 	 else
 	    slicer = __rubber_expansion(slicer)
 	    if slicer.length != rank
@@ -380,7 +382,7 @@ module NumRu
 		  lost.push( ax )
 	       end
 	    end
-	    grid = Grid.new( *axes )
+	    grid = self.class.new( *axes )
 	    grid.set_lost_axes( lost ) if lost.length != 0
 	    grid
 	 end
@@ -449,7 +451,7 @@ module NumRu
 	 dim = dim_index(dim_or_dimname)
 	 axes = @axes.dup
 	 axes.delete_at(dim)
-	 Grid.new( *axes )
+	 self.class.new( *axes )
       end
 
       def change_axis(dim, axis)
@@ -461,7 +463,7 @@ module NumRu
 	    lost.push(axis) if axis.is_a?(String)
 	    axes.delete_at(dim)
 	 end
-	 Grid.new( *axes ).add_lost_axes( lost )
+	 self.class.new( *axes ).add_lost_axes( lost )
       end
 
       def change_axis!(dim_or_dimname, axis)
@@ -484,7 +486,7 @@ module NumRu
 	 else
 	    # do nothing
 	 end
-	 Grid.new( *axes )
+	 self.class.new( *axes )
       end
 
       def insert_axis!(dim_or_dimname, axis)
@@ -508,7 +510,7 @@ module NumRu
 	for i in 0...rank
 	  axes[i] = @axes[dims[i]]
 	end
-	grid = Grid.new(*axes)
+	grid = self.class.new(*axes)
 	grid.set_lost_axes( lost_axes )
 	grid
       end
