@@ -285,7 +285,7 @@ module NumRu
 	    if step == 1
 	       first..last
 	    else
-	       {first..last,step}
+	       {(first..last) => step}
 	    end
 	 end
       end
@@ -299,18 +299,18 @@ end
 
 if $0 == __FILE__
   include NumRu
-   m =  SubsetMapping1D.new(40,{1..-2,2})
+   m =  SubsetMapping1D.new(40,{(1..-2) => 2})
    p '%%',m.slicer,m.regular,m.collapsed,m.first,m.length,m.step,m.index_array
-   m2 =  SubsetMapping1D.new(10,{1..9,4})
+   m2 =  SubsetMapping1D.new(10,{(1..9) => 4})
    print "+++ #{m.map(0)} #{m.map(1)} #{m.imap(0)} #{m.imap(1)}\n"
    p '##',m2.slicer,m2.regular,m2.collapsed,m2.first,m2.length,m2.step,m2.index_array
    m = m.composite(m2)
    p '%%',m.slicer,m.regular,m.collapsed,m.first,m.length,m.step,m.index_array
 
    print "########\n"
-   m =  SubsetMapping1D.new(40,{-2..1,-2})
+   m =  SubsetMapping1D.new(40,{(-2..1) => -2})
    p '%%',m.slicer,m.regular,m.collapsed,m.first,m.length,m.step,m.index_array
-   m2 =  SubsetMapping1D.new(10,{1..9,4})
+   m2 =  SubsetMapping1D.new(10,{(1..9) => 4})
    print "+++ #{m.map(0)} #{m.map(1)} #{m.imap(0)} #{m.imap(1)}\n"
    p '##',m2.slicer,m2.regular,m2.collapsed,m2.first,m2.length,m2.step,m2.index_array
    m = m.composite(m2)
