@@ -955,6 +955,8 @@ These methods returns a NArray (not a VArray).
             result = self.val.#{f}(*args)
             if result.is_a?(NArray) || result.is_a?(NArrayMiss)
 	       VArray.new(result , self.attr_copy, self.name )
+            elsif result.nil?
+               result
             else
 	       UNumeric[result, units]    # used to be 'result' (not UNumeric)
             end
