@@ -6,12 +6,17 @@
 require "numru/netcdf"
 require "numru/gphys/varray"
 require "numru/gphys/attributenetcdf"
+require "numru/gphys/netcdf_convention"
 
 module NumRu
 
+   ###############################
+
+   ### /// for write buffering -->
+
    class NetCDFVar
      alias current_put put
-   end
+   end              # class NetCDFVar
 
    module NetCDFDeferred
       # To be "extend"ed in a NetCDF object to add simgular methods
@@ -53,9 +58,9 @@ module NumRu
 	 enddef
 	 super
       end
-   end
+   end        # module NetCDFDeferred
 
-   ####################
+   #######
 
    module NetCDFVarDeferred
       # To be "extend"ed in a NetCDFVar object to add simgular methods
@@ -113,6 +118,8 @@ module NumRu
       end
    end
 
+   ### <-- for write buffering ///
+
    ####################################################################
 
    class VArrayNetCDF < VArray
@@ -128,6 +135,8 @@ module NumRu
 	 @ary.extend(NetCDFVarDeferred)
 	 @ary.file.extend(NetCDFDeferred)
 	 @attr = AttributeNetCDF.new(aNetCDFVar)
+	 @convention = NetCDF_Conventions.find(aNetCDFVar.file)
+	 extend( @convention::VArray_Mixin )
       end
 
       def inspect
@@ -138,6 +147,14 @@ module NumRu
 	 ## < redefined class methods > ##
 
          def new2(file, name, ntype, dimensions, vary=nil)
+	   dimensions = dimensions.collect{|dim| 
+	     if dim.is_a?(String)
+	       # specification by name is available for existing dimensions
+	       file.dim(dim) || raise("dimension "+dim+" is not in "+file.path)
+	     else
+	       dim
+	     end
+	   }
 	    va = new( file.def_var(name, ntype, dimensions) )
             if vary
 	      vary.att_names.each{|name| va.set_att(name, vary.get_att(name))}
@@ -145,6 +162,8 @@ module NumRu
             va
          end
 
+	 alias def_var new2
+
 	 ## < additional class methods > ##
 
 	 def write(file, vary, rename=nil, dimnames=nil)
@@ -281,8 +300,13 @@ module NumRu
 	 @ary.file
       end
 
-   end
-end
+      ## Not needed, so far:
+      #def convention
+      #  @convention
+      #end
+
+   end     # class VArrayNetCDF
+end     # module NumRu
 
 ###########################################################
 ### < test >
@@ -297,6 +321,7 @@ if $0 == __FILE__
    x.val = 10
    v = VArrayNetCDF.new2(file,"v","sfloat",dims)
    v.set_att("units","m/s")
+   v_ = VArrayNetCDF.new2(file,"v_","sfloat",["x","t"])
    # file.enddef  # (no need to call this)
    v[0..-1,0] = 99.0
    v[0..-1,1] = 66.0
